@@ -9,7 +9,7 @@ import numpy as np
 
 import nibabel as nif
 from nipype.testing import (assert_equal, assert_not_equal,
-                            assert_raises, parametric, skipif)
+                            assert_raises, skipif)
 from nipype.interfaces.base import TraitError
 
 import nipype.interfaces.freesurfer as fs
@@ -161,6 +161,36 @@ def test_surfxfm():
     # Clean up
     clean_directory(cwd, oldwd)
 
+@skipif(no_freesurfer)
+def test_applymask():
+    masker = fs.ApplyMask()
+
+    filelist, testdir, origdir = create_files_in_directory()
+
+    # Test underlying command
+    yield assert_equal, masker.cmd, "mri_mask"
+
+    # Test exception with mandatory args absent
+    yield assert_raises, ValueError, masker.run
+    for input in ["in_file", "mask_file"]:
+        indict = {input:filelist[0]}
+        willbreak = fs.ApplyMask(**indict)
+        yield assert_raises, ValueError, willbreak.run
+    
+    # Now test a basic command line
+    masker.inputs.in_file = filelist[0]
+    masker.inputs.mask_file = filelist[1]
+    outfile = os.path.join(testdir, "a_masked.nii")
+    yield assert_equal, masker.cmdline, "mri_mask a.nii b.nii %s"%outfile
+    # Now test that optional inputs get formatted properly
+    masker.inputs.mask_thresh = 2
+    yield assert_equal, masker.cmdline, "mri_mask -T 2.0000 a.nii b.nii %s"%outfile
+    masker.inputs.use_abs = True
+    yield assert_equal, masker.cmdline, "mri_mask -T 2.0000 -abs a.nii b.nii %s"%outfile
+
+    # Now clean up
+    clean_directory(testdir, origdir)
+
 @skipif(no_freesurfer)
 def test_surfshots():
 
@@ -197,10 +227,13 @@ def test_surfshots():
     yield assert_equal, fotos.cmdline, "tksurfer fsaverage lh pial -tcl other.tcl"
 
     # Test that the interface crashes politely if graphics aren't enabled
-    hold_display = os.environ["DISPLAY"]
-    del os.environ["DISPLAY"]
-    yield assert_raises, RuntimeError, fotos.run
-    os.environ["DISPLAY"] = hold_display
-
+    try:
+        hold_display = os.environ["DISPLAY"]
+        del os.environ["DISPLAY"]
+        yield assert_raises, RuntimeError, fotos.run
+        os.environ["DISPLAY"] = hold_display
+    except KeyError:
+        pass
+    
     # Clean up our mess
     clean_directory(cwd, oldwd)
